/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The Software and/or source code cannot be copied in whole and
sold without meaningful modification for a profit.

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with
the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
    File        : adrtb_buildimport.p
    Purpose     : Procedure using RTB API and RTBAutomation API to build an import control list

    Syntax      : run appSolutions/appSolute/RTB/Automation/adrtb_buildimport.p -param InstallPath,user,password,Wspace,SourceWorkspaces,Pmods,Tasks,ReleaseNum,HigherVerOnly,CurrReleaseOnly,IncludeAll

    Description : appSolute Deployment Procedure for creating release in RTB using RTB and ADRTB API

    Created     : Fri Nov 14 09:53:22 CET 2008
    Notes       :
                11/2013 Thomas Hansen
                Restructured code to use ABL structured error handling better.
  ----------------------------------------------------------------------*/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

/* ***************************  Definitions  ************************** */
DEFINE OUTPUT PARAMETER pcImportAnalaysisReport      AS CHARACTER NO-UNDO.
DEFINE OUTPUT PARAMETER plImportTableCreated         AS LOGICAL   NO-UNDO.

/* Standard variable definitions for RTB API usage */
{appSolutions/appSolute/RTB/Automation/arat_base_props.i}

DEFINE        VARIABLE gcSourceWorkspaces           AS CHARACTER NO-UNDO.
DEFINE        VARIABLE gcImportPmods                AS CHARACTER NO-UNDO.
DEFINE        VARIABLE gcImportTasks                AS CHARACTER NO-UNDO.
DEFINE        VARIABLE giImportReleaseNum           AS INTEGER   NO-UNDO.
DEFINE        VARIABLE glImportHigherVerOnly        AS LOGICAL   NO-UNDO.
DEFINE        VARIABLE glImportCurrReleaseOnly      AS LOGICAL   NO-UNDO.
DEFINE        VARIABLE glCompleteTasksOnly          AS LOGICAL   NO-UNDO INITIAL TRUE.
DEFINE        VARIABLE glImportIncludeAll           AS LOGICAL   NO-UNDO.
DEFINE        VARIABLE glCreateImportReport         AS LOGICAL   NO-UNDO.
DEFINE        VARIABLE glDeleteExistsingImportTable AS LOGICAL   NO-UNDO.
DEFINE        VARIABLE gcImportFilterSettings       AS CHARACTER NO-UNDO.

{rtb/i/rtb_super.i}

ASSIGN
   giParamsExpected = 19
   gcLogFile        = "logs/adrtb_buildimport.log":U.

RUN initialize.

RUN processWorkspaces.

RETURN "0". /* Success */

CATCH Err AS Progress.Lang.Error :
   ASSIGN
      cErrors = " ***":U + Err:GetMessage(1).

   IF VALID-OBJECT(oRTBAutomationApi) THEN
      oRTBAutomationApi:displayMessage(cErrors, YES, YES).
   ELSE
      MESSAGE
         cErrors
         VIEW-AS ALERT-BOX ERROR BUTTONS OK.

   RUN shutdown.

   RETURN "1". /* Failure */

END CATCH.

FINALLY:
   RUN shutdown.
END FINALLY.


/* **********************  Internal Procedures  *********************** */

PROCEDURE buildImportList:
   /*------------------------------------------------------------------------------
           Purpose:
           Notes:
   ------------------------------------------------------------------------------*/
   DEFINE INPUT  PARAMETER pcWorkspace AS CHARACTER NO-UNDO.

   DEFINE VARIABLE cError            AS CHARACTER NO-UNDO.
   DEFINE VARIABLE cMessage          AS CHARACTER NO-UNDO.
   DEFINE VARIABLE lImportListExists AS LOGICAL   NO-UNDO.
   DEFINE VARIABLE iNumRecords       AS INTEGER   NO-UNDO.
   DEFINE VARIABLE cImportReport     AS CHARACTER NO-UNDO.

   ASSIGN
      cMessage = SUBSTITUTE("   Building workspace import list for workspace : &1", pcWorkspace).
   oRTBAutomationApi:displayMessage(cMessage, YES, YES).

   ASSIGN
      gcImportFilterSettings = "   Import settings " + "~n":U +
                    "   =================" + "~n":U +
                    SUBSTITUTE("   Workspace           : &1~n", pcWorkspace) +
                    SUBSTITUTE("   Source workspaces   : &1~n", gcSourceWorkspaces) +
                    SUBSTITUTE("   Product modules     : &1~n", gcImportPmods) +
                    SUBSTITUTE("   Tasks               : &1~n", gcImportTasks) +
                    SUBSTITUTE("   Releases            : &1~n", giImportReleaseNum) +
                    SUBSTITUTE("   Higher versions only: &1~n", glImportHigherVerOnly) +
                    SUBSTITUTE("   Current release only: &1~n", glImportCurrReleaseOnly) +
                    SUBSTITUTE("   Complete tasks only : &1~n", glCompleteTasksOnly).

   oRTBAutomationApi:displayMessage(gcImportFilterSettings, YES, YES).

   lImportListExists = oRTBAutomationApi:checkImportTableExists(pcWorkspace, OUTPUT iNumRecords).

   IF lImportListExists THEN
   DO:
      IF glDeleteExistsingImportTable THEN
      DO:
         ASSIGN
            cMessage = "   Import table exists already. Deleting existing workspace import table...".
         oRTBAutomationApi:displayMessage(cMessage, YES, YES).

         oRTBAutomationApi:deleteWorkspaceImportList(
            INPUT  pcWorkspace,
            OUTPUT cError).

         IF cError NE "":U THEN
            UNDO, THROW NEW Progress.Lang.AppError(cError, 0).

         /* Reset lImportListExists as we are going to use it again */
         ASSIGN
            lImportListExists = FALSE.
      END.
      ELSE
      DO:
         ASSIGN
            cError = "   *** Import control table exists already. Cannot build new import control table until this has been deleted.".
         oRTBAutomationApi:displayMessage(cMessage, YES, TRUE).

         UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
      END.
   END.

   ASSIGN
      cMessage = "   Building workspace import table ...".
   oRTBAutomationApi:displayMessage(cMessage, YES, TRUE).

   oRTBAutomationApi:buildWorkspaceImportList (
      INPUT  pcWorkspace,
      INPUT  gcSourceWorkspaces,
      INPUT  gcImportPmods,
      INPUT  gcImportTasks,
      INPUT  giImportReleaseNum,
      INPUT  glImportHigherVerOnly,
      INPUT  glImportCurrReleaseOnly,
      INPUT  glCompleteTasksOnly,
      OUTPUT cError).

   IF cError NE "":U THEN
   DO:
      ASSIGN
         cError = "*** RTB Build import table error: ~n" + cError.
      /*      oRTBAutomationApi:displayMessage(cError, YES, TRUE).*/
      IF cError NE "":U THEN
         UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
   END.
   ELSE
   DO:
      ASSIGN
         cMessage = "   Building workspace import table completed".
      oRTBAutomationApi:displayMessage(cMessage, YES, glDebug).
   END.

   lImportListExists = oRTBAutomationApi:checkImportTableExists(pcWorkspace, OUTPUT iNumRecords).

   IF lImportListExists THEN
   DO:
      ASSIGN
         plImportTableCreated = TRUE
         cMessage             = SUBSTITUTE("   Import control table created. Number of records in import table: &1 ", STRING(iNumRecords, ">>,>>9":U)).
      oRTBAutomationApi:displayMessage(cMessage, YES, YES).

      IF glImportIncludeAll = TRUE THEN
      DO:
         ASSIGN
            cMessage = "   Setting 'include all' on import table...".
         oRTBAutomationApi:displayMessage(cMessage, YES, YES).

         /* Make sure all entries in the import table are set to be included.
            Note this is a dangerous thing to have set as a default - so use with caution.  */
         IF glImportIncludeAll THEN
            oRTBAutomationApi:rtbIncludeAllImports(INPUT pcWorkspace).
      END.

      IF glCreateImportReport THEN
      DO:
         oRTBAutomationApi:createImportAnalysisReport (
            INPUT  pcWorkspace,
            INPUT  SUBSTITUTE("&1/&2/&3", oRTBAutomationApi:RtbReportBaseDir,"Imports":U, oRTBAutomationApi:RtbWorkspaceId),
            INPUT  gcImportFilterSettings,
            OUTPUT cImportReport,
            OUTPUT cError).

         IF cError NE "":U THEN
            UNDO, THROW NEW Progress.Lang.AppError(cError, 0).

         ASSIGN
            cMessage = "   Import analysis report created and written to file : ":U + cImportReport + "...".
         oRTBAutomationApi:displayMessage(cMessage, YES, TRUE).

         ASSIGN
            pcImportAnalaysisReport = REPLACE(cImportReport, "~\":U, "/":U).
      END.
   END.
   ELSE
   DO:
      ASSIGN
         plImportTableCreated = FALSE
         cMessage             = "   No import table created.".
      oRTBAutomationApi:displayMessage(cMessage, YES, TRUE).
   END.

   RETURN.

   CATCH Err AS Progress.Lang.Error :
      IF oRTBAutomationApi:DebugOn THEN
         oRTBAutomationApi:DisplayMessage(SUBSTITUTE("*** [Debug &1] : &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).
      UNDO, THROW Err.
   END CATCH.

END PROCEDURE.

PROCEDURE initialize:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    {appSolutions/appSolute/RTB/Automation/arat_init_rtbapis.i}

   RETURN.

   CATCH Err AS Progress.Lang.Error:
      IF oRTBAutomationApi:DebugOn THEN
         oRTBAutomationApi:DisplayMessage(Err:GetMessage(1), TRUE, TRUE).

      UNDO, THROW Err.
   END CATCH.

   FINALLY:
      IF VALID-OBJECT(oFileHelper) THEN
         DELETE OBJECT oFileHelper.
   END FINALLY.

END PROCEDURE.

PROCEDURE processWorkspaces:
   /*------------------------------------------------------------------------------
           Purpose:
           Notes:
   ------------------------------------------------------------------------------*/
   DEFINE VARIABLE i          AS INTEGER   NO-UNDO.
   DEFINE VARIABLE cError     AS CHARACTER NO-UNDO.
   DEFINE VARIABLE cMessage   AS CHARACTER NO-UNDO.
   DEFINE VARIABLE cWorkspace AS CHARACTER NO-UNDO.

    ASSIGN
        cMessage = SUBSTITUTE("  Number of workspaces specified: &2, Minimum allowed workspaces: 1, Maximum allowed workspaces: &1", giMaxWorkspaces, NUM-ENTRIES(gcWorkspaceList)).
    oRTBAutomationApi:displayMessage(cMessage, TRUE, TRUE).

    IF NUM-ENTRIES(gcWorkspaceList) > giMaxWorkspaces THEN
    DO:
        ASSIGN
            cMessage = SUBSTITUTE("  Number of workspaces specified exceeds the maximum of &1.", giMaxWorkspaces).

        UNDO, THROW NEW Progress.Lang.AppError(cMessage, 0).
    END.

   IF gcWorkspaceList NE "":U THEN
   DO:
      IF NUM-ENTRIES(gcWorkspaceList) > 1 THEN
      DO:
         ASSIGN
            gcWorkspaceList = TRIM(gcWorkspaceList)
            gcWorkspaceList = TRIM(gcWorkspaceList, ",":U)
            gcWorkspaceList = REPLACE(gcWorkspaceList, ",,",",":U)
            gcWorkspaceList = REPLACE(gcWorkspaceList, "~n","":U) NO-ERROR.

         ASSIGN
            cMessage = SUBSTITUTE("  Processing workspaces : &1...",  gcWorkspaceList).
         oRTBAutomationApi:displayMessage(cMessage, YES, YES).
      END.

      DO i = 1 TO NUM-ENTRIES(gcWorkspaceList) ON ERROR UNDO, NEXT:

         ASSIGN
            cMessage   = "":U
            cWorkspace = TRIM(ENTRY(i, gcWorkspaceList))
            cWorkspace= TRIM(cWorkspace, ",":U).

         IF cWorkspace = "":U THEN
            NEXT.

         IF i > 1 THEN
            ASSIGN
               cMessage = "  /*****************************************/".

         ASSIGN
            cMessage = cMessage + SUBSTITUTE("~n  Processing workspace: &1...",  cWorkspace) + "~n  /*****************************************/".
         oRTBAutomationApi:displayMessage(cMessage, YES, YES).

         IF oRTBAutomationApi:RTBProxyHelper:rtbWorkspaceExists(cWorkspace) THEN
         DO:
            oRTBAutomationApi:setRTBWorkspaceID(cWorkspace).
            RUN buildImportList (cWorkspace).
         END.
         ELSE
         DO:
            ASSIGN
               cError = SUBSTITUTE("  *** Workspace '&1' not valid. Cannot build import list!", cWorkspace).
            oRTBAutomationApi:displayMessage(cError, YES, YES).
         END.

         CATCH err AS Progress.Lang.Error :
            ASSIGN
               cMessage = SUBSTITUTE(" *** Import build processing error for workspace '&1': &2", cWorkspace, err:GetMessage(1)).
            oRTBAutomationApi:displayMessage(cMessage, YES, YES).

            ASSIGN
               glProcessFailed = TRUE.
         END CATCH.
      END.
   END.

   IF glProcessFailed THEN
      UNDO, THROW NEW Progress.Lang.AppError("Workspace deployment processing failures occured. Overall processing failed.", 0).

   RETURN.

   CATCH Err AS Progress.Lang.Error :
      IF oRTBAutomationApi:DebugOn THEN
         oRTBAutomationApi:DisplayMessage(SUBSTITUTE("*** [Debug &1] : &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).
      UNDO, THROW Err.
   END CATCH.
END PROCEDURE.

PROCEDURE readparams:
   /*------------------------------------------------------------------------------
    Purpose:
    Notes:
   ------------------------------------------------------------------------------*/

   DEFINE VARIABLE oListHelper AS appSolutions.appSolute.Utils.ListHelper NO-UNDO.

   IF ghBuf:AVAILABLE THEN
   DO:
      ASSIGN
         glDebug                      = ghBuf::BuildDebug
         gcRTBInstallPath             = ghBuf::RtbInstallPath
         gcUser                       = ghBuf::RtbUser
         gcPassword                   = ghBuf::RtbPassword
         gcWorkspaceList              = ghBuf::RtbImportWspaceId
         gcSourceWorkspaces           = ghBuf::RtbImportSourceWorkspaces
         giMaxWorkspaces              = ghBuf::RtbImportMaxWorkspaces
         gcImportPmods                = ghBuf::RtbImportPmods
         gcImportTasks                = ghBuf::RtbImportTaskList
         giImportReleaseNum           = ghBuf::RtbImportReleaseNum
         glImportHigherVerOnly        = ghBuf::RtbImportHigherVerOnly
         glImportCurrReleaseOnly      = ghBuf::RtbImportCurrReleaseOnly
         glImportIncludeAll           = ghBuf::RtbImportIncludeAll
         glCompleteTasksOnly          = ghBuf::RtbImportCompleteTasksOnly
         glDeleteExistsingImportTable = ghBuf::RtbImportDeleteExistingImportTable
         glCreateImportReport         = ghBuf::RtbImportReportAfterBuild
         gcAppServerConnectString     = ghBuf::RtbAppServerConnection
         glUseAppServer               = ghBuf::RtbUseAppServer
         gcReportBaseDir              = ghBuf::RtbReportBaseDir.
   END.

   gcImportTasks = oRTBAutomationApi:cleanupCsvList(gcImportTasks).
   gcWorkspaceList = oRTBAutomationApi:cleanupCsvList(gcWorkspaceList).

   IF gcImportTasks BEGINS "$":U THEN
      ASSIGN
         gcImportTasks = "":U.

   IF gcImportTasks = "*":U THEN
      ASSIGN
         gcImportTasks = "":U.

   IF gcImportTasks = "0":U THEN
      ASSIGN
         gcImportTasks = "":U.

   IF gcSourceWorkspaces = "*" THEN
      ASSIGN
         gcSourceWorkspaces = "":U.

   IF gcImportPmods = "*":U THEN
      ASSIGN
         gcImportPmods = "":U.

   IF gcImportTasks BEGINS "File:":U THEN
   DO ON ERROR UNDO:
      oListHelper = NEW appSolutions.appSolute.Utils.ListHelper().

      gcImportTasks = oListHelper:getListFromFile(REPLACE(gcImportTasks, "File:":U, "":U)).

      CATCH err AS Progress.Lang.Error :
         UNDO, THROW NEW Progress.Lang.AppError(err:GetMessage(1), 0).
      END CATCH.
   END.

   RETURN.

END PROCEDURE.

PROCEDURE shutdown:
   /*------------------------------------------------------------------------------
           Purpose:
           Notes:
   ------------------------------------------------------------------------------*/
   oRTBAutomationApi:stopAPIs().

   IF VALID-OBJECT(oRTBAutomationApi) THEN
      DELETE OBJECT oRTBAutomationApi.

   RETURN.

END PROCEDURE.



