/*********************************************************************
 Copyright (C) 2012-present by nodeable limited ("nodeable") -
 www.nodeable.io.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The Software and/or source code cannot be copied in whole and
sold without meaningful modification for a profit.

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with
the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
    File        : adrtb_wsintegritycheck.p
    Purpose     : Procedure using RTB API and ADRTB API to do a Verson Integrity Check of objects in a workspace

    Syntax      : run appSolutions/appSolute/RTB/Automation/adrtb_buildimport.p
                  -param InstallPath,user,password,Wspace,ObjectFilter

    Description : appSolute Deployment Procedure for running Test Version Integrity on objects in a workspace

    Created     : 02/10/2012
    Notes       :
                11/2013 Thomas Hansen
                Restructured code to use ABL structured error handling better.
  ----------------------------------------------------------------------*/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

/* ***************************  Definitions  ************************** */
DEFINE OUTPUT PARAMETER pcLogFiles AS CHARACTER NO-UNDO.

/* Standard variable definitions for RTB API usage */
{appSolutions/appSolute/RTB/Automation/arat_base_props.i}

/* Specific variables for this process */
DEFINE VARIABLE giEventCounter      AS INTEGER   NO-UNDO.
DEFINE VARIABLE gcObjectFilter      AS CHARACTER NO-UNDO.
DEFINE VARIABLE glDisplayWIPObjects AS LOGICAL   NO-UNDO INITIAL TRUE.

ASSIGN
    giParamsExpected = 10
    glDebug          = FALSE
    gcLogFile        = "logs/adrtb_wsintegritycheck.log":U.

RUN initialize.

RUN WsIntegrityCheck.

RETURN "0". /* Success */

CATCH Err AS Progress.Lang.Error :
    ASSIGN
        cErrors = SUBSTITUTE(" *** &1":U, Err:GetMessage(1)).

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        oRTBAutomationApi:displayMessage(cErrors, YES, YES).
    ELSE
        MESSAGE
            cErrors
            VIEW-AS ALERT-BOX ERROR BUTTONS OK.

    RUN shutdown.

    RETURN "1".

END CATCH.

FINALLY:
    RUN shutdown.
END FINALLY.
/* **********************  Internal Procedures  *********************** */

PROCEDURE initialize:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    {appSolutions/appSolute/RTB/Automation/arat_init_rtbapis.i}

    RETURN.

    CATCH LangError AS Progress.Lang.Error:
        UNDO, THROW LangError.
    END CATCH.

END PROCEDURE.

PROCEDURE readParams:
    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    IF ghBuf:AVAILABLE THEN
    DO:
        ASSIGN
            glDebug                  = ghBuf::BuildDebug
            gcRTBInstallPath         = ghBuf::RtbInstallPath
            gcUser                   = ghBuf::RtbUser
            gcPassword               = ghBuf::RtbPassword
            gcWorkspaceList          = ghBuf::RtbWsCheckWspaceId
            gcObjectFilter           = ghBuf::RtbWsCheckObjectFilter
            glDisplayWIPObjects      = ghBuf::RtbWscheckDisplayWIPObjects
            gcAppServerConnectString = ghBuf::RtbAppServerConnection
            glUseAppServer           = ghBuf::RtbUseAppServer
            gcReportBaseDir          = ghBuf::RtbReportBaseDir
        NO-ERROR.
    END.

    RETURN.

END PROCEDURE.

PROCEDURE WsIntegrityCheck:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cWorkspace            AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cError                AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cMessage              AS CHARACTER NO-UNDO.
    DEFINE VARIABLE iObjectsChecked       AS INTEGER   NO-UNDO.
    DEFINE VARIABLE iObjectErrors         AS INTEGER   NO-UNDO.
    DEFINE VARIABLE iWIPObjects           AS INTEGER   NO-UNDO.
    DEFINE VARIABLE cFullPathname         AS CHARACTER NO-UNDO.
    DEFINE VARIABLE hWorkspaceObjectsData AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hQry                  AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hWorkspaceObjectBuf   AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hVersionData          AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hVerBuf               AS HANDLE    NO-UNDO.
    DEFINE VARIABLE i                     AS INTEGER   NO-UNDO.
    DEFINE VARIABLE lcResults             AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE cLogsDir              AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cLogFile              AS CHARACTER NO-UNDO.

    DO i = 1 TO NUM-ENTRIES(gcWorkspaceList) ON ERROR UNDO, NEXT:
        ASSIGN
            cWorkspace = ENTRY(i, gcWorkspaceList).

        IF i > 1 THEN
            ASSIGN
                cMessage = "  /*****************************************/".

        ASSIGN
            cMessage = cMessage + SUBSTITUTE("~n  Processing workspace: &1...",  cWorkspace) + "~n  /*****************************************/".
        oRTBAutomationApi:displayMessage(cMessage, YES, YES).

        IF oRTBAutomationApi:ValidWorkspace(cWorkspace) THEN
        DO ON ERROR UNDO:
            ASSIGN
                cMessage = SUBSTITUTE("~n   Checking integrity of workspace: &1", cWorkspace).
            oRTBAutomationApi:displayMessage(cMessage, YES, YES).

            oRTBAutomationApi:checkLogsDir(
                INPUT  "IntegrityCheck",
                INPUT  cWorkspace,
                INPUT  TRUE,
                OUTPUT cLogsDir).

            ASSIGN
                cLogFile = oRTBAutomationApi:buildOutputFilename (cWorkspace, "IntegrityCheckReport":U).

            oRTBAutomationApi:setRTBWorkspaceID(cWorkspace).
            oRTBAutomationApi:checkWorkspaceIntegrity(
                INPUT  cWorkspace,
                INPUT  gcObjectFilter,
                INPUT  glDisplayWIPObjects,
                OUTPUT lcResults).

            IF lcResults NE "":U THEN
            DO:
                ASSIGN
                    cLogFile = SUBSTITUTE("&1/&2", cLogsDir, cLogFile).

                COPY-LOB FROM lcResults TO FILE cLogFile NO-CONVERT.

                ASSIGN
                    cMessage = SUBSTITUTE("   Integrity check results written to : &1", cLogFile).
                oRTBAutomationApi:displayMessage(cMessage, YES, YES).

                ASSIGN
                    pcLogFiles = pcLogFiles + ",":U + REPLACE(cLogFile, "~\":U, "/":U)
                    pcLogFiles = REPLACE(pcLogFiles, "~\", "/":U).
            END.

            CATCH err AS Progress.Lang.Error :
                ASSIGN
                    cMessage = SUBSTITUTE(" *** Workspace integrity check processing error for workspace '&1': &2", cWorkspace, err:GetMessage(1)).
                oRTBAutomationApi:displayMessage(cMessage, YES, YES).
            END CATCH.
        END.
    END.

    ASSIGN
        pcLogFiles = TRIM(pcLogFiles, ",":U).

    ASSIGN
        cMessage = "   Deployment processing complete.".
    oRTBAutomationApi:displayMessage(cMessage, TRUE, TRUE).

    RETURN.

    CATCH Err AS Progress.Lang.Error :
        IF oRTBAutomationApi:DebugOn THEN
            oRTBAutomationApi:DisplayMessage(SUBSTITUTE("*** [Debug &1] : &2", "{&line-number} " + PROGRAM-NAME(1), err:GetMessage(1)), YES, YES).
        UNDO, THROW Err.
    END CATCH.

END PROCEDURE.

PROCEDURE shutdown:
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    oRTBAutomationApi:stopAPIs().

    IF VALID-OBJECT(oRTBAutomationApi) THEN
        DELETE OBJECT oRTBAutomationApi.

    RETURN.

END PROCEDURE.


