/*********************************************************************
* Copyright (C) 2012-present by nodeable limited ("nodeable")        *
* www.nodeable.io.com and other contributors as listed below.        *
* All Rights Reserved.                                               *
*                                                                    *
*  Software is distributed on an "AS IS", WITHOUT WARRANTY OF ANY    *
*  KIND, either express or implied.                                  *
*                                                                    *
* Contributors:                                                      *
*  appSolutions a/s                                                  *
*  Customized version of standard compiler library from              *
*  Roundtable 11.5                                                   *
*********************************************************************/
/*
 Customized copy of standard rtb/p/rtb_compiler.p
 Roundtable compiler library

 This custom version includes customizations to show an OK status on objects as they are compiled.
 This is to get a compile listing that shows which objects have been compiled and not just the ones with errors.

 Notes:
  Two public procedures are available: rtb_compile_object and
  rtb_selective_compile.

  To compile, a call is make to rtbCompilePre.p to get a table of objects
  requiring compilation. The table is then split into client-side and server-
  side compiles.  Each table is then processed as follows:

  A compile-batch is created based on the compile batch size. That batch is
  then passed as input-output to rtbCompileObjects.p. Each object that
  sucessfully compiled is added to a "safe-list" batch which is then passed
  as input-output to rtbCompilePost.p.  The xref results are examined and any
  errors are written to the compile log (located in the RTB session temp-dir.
  Those objects which did not sucessfully compile are written to the compile
  log.

  After each batch, a check is made to see if the compile has been cancelled
  and exits accordingly.

  Both the rtb_compile_object and rtb_selective_compile procedures can either
  return a CHR(1) delimited error message or an error message only.  If the
  error message is CHR(1) delimited, the 2nd entry will be the path to the
  error file.
*/
CREATE WIDGET-POOL.

DEFINE INPUT  PARAMETER phAppServerHandle AS HANDLE NO-UNDO.

{rtb/i/rtb_super.i}
{rtb/i/rtb_ttcomp.i}

FUNCTION getTableCount RETURNS INTEGER PRIVATE (INPUT phTT AS HANDLE) FORWARD.

DEFINE TEMP-TABLE ttCompBatch LIKE ttCompObj.
DEFINE TEMP-TABLE ttClientCompilables LIKE ttCompObj.
DEFINE TEMP-TABLE ttServerCompilables LIKE ttCompObj.

DEFINE VARIABLE cObject                    AS CHARACTER NO-UNDO.
DEFINE VARIABLE cError                     AS CHARACTER NO-UNDO.
DEFINE VARIABLE cErrorFile                 AS CHARACTER NO-UNDO.
DEFINE VARIABLE cWorkspace                 AS CHARACTER NO-UNDO.
DEFINE VARIABLE hAsConn                    AS HANDLE    NO-UNDO.
DEFINE VARIABLE hServer                    AS HANDLE    NO-UNDO.
DEFINE VARIABLE hFileLW                    AS HANDLE    NO-UNDO.
DEFINE VARIABLE hDbParam                   AS HANDLE    NO-UNDO.
DEFINE VARIABLE iCompSize                  AS INTEGER   NO-UNDO.
DEFINE VARIABLE iBatchSize                 AS INTEGER   NO-UNDO INITIAL 10.
DEFINE VARIABLE lHasCompileMsg             AS LOGICAL   NO-UNDO.
DEFINE VARIABLE lHasXrefError              AS LOGICAL   NO-UNDO.
DEFINE VARIABLE lCompileCancelled          AS LOGICAL   NO-UNDO.
DEFINE VARIABLE lStopOnError               AS LOGICAL   NO-UNDO.
DEFINE VARIABLE lIgnoreWarn                AS LOGICAL   NO-UNDO INITIAL NO.
DEFINE VARIABLE lTaskComp                  AS LOGICAL   NO-UNDO INITIAL NO.
DEFINE VARIABLE lTaskRcode                 AS LOGICAL   NO-UNDO INITIAL NO.
DEFINE VARIABLE lSelectiveComp             AS LOGICAL   NO-UNDO INITIAL NO.

DEFINE VARIABLE glOutputSuccessfulCompiles AS LOGICAL   NO-UNDO INITIAL NO.
DEFINE VARIABLE glHasCompileResults        AS LOGICAL   NO-UNDO INITIAL NO.

/* 10/07/2013 - Thomas Hansen / appSolutions
Here we want to control the AppServer connection ourselves, with a new input parameter */
IF VALID-HANDLE(phAppServerHandle) THEN
DO:
    ASSIGN
        hServer = phAppServerHandle.

    hFileLW = fnRtbStartSuperProc("rtb/p/rtb_fileloadwrite.p", THIS-PROCEDURE).
END.
ELSE
    hServer = SESSION:HANDLE.

hDbparam = fnRtbStartSuperSdo("rtb_u_dbparam.w", THIS-PROCEDURE, NO).


ASSIGN
    glOutputSuccessfulCompiles = TRUE.

RETURN.



/* **********************  Internal Procedures  *********************** */


PROCEDURE rtb_compile_object:
    /*----------------------------------------------------------------
     Purpose:    Compile a single object
     Parameters: INPUT
                   pcWorkspace - workspace id
                   piTaskNum   - task number
                   pcObjType   - object type
                   pcObject    - object name
                   pcPmod      - product module
                   plXref      - yes = compile with xref
                   plListing   - yes = save compile and xref listings
               plLab       - yes = lab compile
     Notes:
    -----------------------------------------------------------------*/
    DEFINE INPUT  PARAMETER pcWorkspace AS CHARACTER   NO-UNDO.
    DEFINE INPUT  PARAMETER piTaskNum   AS INTEGER     NO-UNDO.
    DEFINE INPUT  PARAMETER pcObjType   AS CHARACTER   NO-UNDO.
    DEFINE INPUT  PARAMETER pcObject    AS CHARACTER   NO-UNDO.
    DEFINE INPUT  PARAMETER pcPmod      AS CHARACTER   NO-UNDO.
    DEFINE INPUT  PARAMETER plXref      AS LOGICAL     NO-UNDO.
    DEFINE INPUT  PARAMETER plListings  AS LOGICAL     NO-UNDO.
    DEFINE INPUT  PARAMETER plLab       AS LOGICAL     NO-UNDO.

    ASSIGN
        cWorkspace     = pcWorkspace
        lSelectiveComp = NO.

    $COMPILE-OBJECT:
    DO ON ERROR UNDO, RETRY:

        IF RETRY THEN
        DO:
            IF cError = "" THEN
                cError = (IF RETURN-VALUE <> "":U THEN
                    RETURN-VALUE
                    ELSE
                    DYNAMIC-FUNCTION('fnRtbErrorHandler',ERROR-STATUS)).
            RETURN cError.
        END.

        RUN build_compile_list
            (INPUT piTaskNum,
            INPUT "",        /* module */
            INPUT "",        /* object group */
            INPUT pcObjType,
            INPUT pcObject,
            INPUT pcPmod,
            INPUT plXref,
            INPUT plListings,
            INPUT YES,       /* force */
            INPUT NO,        /* selective */
            INPUT NO,        /* ignore warnings */
            INPUT NO,        /* task completion */
            INPUT plLab,
            INPUT NO,          /* exclude dependencies */
            OUTPUT iCompSize). /* number of objects selected */

        RUN process_compile_list (plXref).

        IF lHasCompileMsg THEN
        DO:
            cError = "Compile Error." + CHR(1) + cErrorFile.
            RETURN cError.
        END.

        IF lHasXrefError THEN
        DO:
            cError = "Xref Error." + CHR(1) + cErrorFile.
            RETURN cError.
        END.

    END. /* $compile-object */

END PROCEDURE.

PROCEDURE rtb_selective_compile:
    /*----------------------------------------------------------------
     Purpose:    Perform a selective compilation
     Parameters: INPUT
                   pcWorkpsace   - workspace id
                   piTaskNum     - task number
                   pcModule      - workspace module
                   pcObjGroup    - object group
                   pcObjType     - object type
                   pcObject      - object name
                   pcPmod        - product module, blank for selective compile
                   plXref        - yes = compile with xref
                   plListings    - yes = save xref and listings
                   plForce       - yes = force compilation
                   plTaskComp    - yes = task completion compile
                   plIgnoreWarn  - yes = ignore compile warnings
                   plStopOnError - yes = stop processing on first error
                   plTaskRcode   - yes = r-code for dependents in task dir
               plNoDependencies - yes = don't compile dependencies
     Notes:      This routine is called by the selective compile and
                 task completion processes. For task-completion, a YES
                 is passed via plTaskComp.
    -----------------------------------------------------------------*/
    DEFINE INPUT PARAMETER pcWorkspace   AS CHARACTER NO-UNDO.
    DEFINE INPUT PARAMETER piTaskNum     AS INTEGER   NO-UNDO.
    DEFINE INPUT PARAMETER pcModule      AS CHARACTER NO-UNDO.
    DEFINE INPUT PARAMETER pcObjGroup    AS CHARACTER NO-UNDO.
    DEFINE INPUT PARAMETER pcObjType     AS CHARACTER NO-UNDO.
    DEFINE INPUT PARAMETER pcObject      AS CHARACTER NO-UNDO.
    DEFINE INPUT PARAMETER pcPmod        AS CHARACTER NO-UNDO.
    DEFINE INPUT PARAMETER plXref        AS LOGICAL   NO-UNDO.
    DEFINE INPUT PARAMETER plListings    AS LOGICAL   NO-UNDO.
    DEFINE INPUT PARAMETER plForce       AS LOGICAL   NO-UNDO.
    DEFINE INPUT PARAMETER plTaskComp    AS LOGICAL   NO-UNDO.
    DEFINE INPUT PARAMETER plIgnoreWarn  AS LOGICAL   NO-UNDO.
    DEFINE INPUT PARAMETER plStopOnError AS LOGICAL   NO-UNDO.
    DEFINE INPUT PARAMETER plTaskRcode    AS LOGICAL  NO-UNDO.
    DEFINE INPUT PARAMETER plNoDependencies AS LOGICAL  NO-UNDO.

    DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

    ASSIGN
        cObject        = pcObject
        cWorkspace     = pcWorkspace
        lStopOnError   = plStopOnError
        lIgnoreWarn    = plIgnoreWarn
        lTaskComp      = plTaskComp
        lSelectiveComp = YES
        lTaskRcode     = plTaskRcode.

    $SELECTIVE-COMPILE:
    DO ON ERROR UNDO, RETRY:

        IF RETRY THEN
        DO:
            IF cError = "":U THEN
                cError = (IF RETURN-VALUE <> "":U THEN
                    RETURN-VALUE
                    ELSE
                    DYNAMIC-FUNCTION('fnRtbErrorHandler',ERROR-STATUS)).
            RETURN cError.
        END.

        RUN build_compile_list
            (INPUT piTaskNum,
            INPUT pcModule,
            INPUT pcObjGroup,
            INPUT pcObjType,
            INPUT pcObject,
            INPUT pcPmod,     /* product module, pass a blank  */
            INPUT plXref,
            INPUT plListings,
            INPUT plForce,
            INPUT YES,         /* selective compile */
            INPUT plIgnoreWarn,
            INPUT plTaskComp,
            INPUT NO,          /* lab not supported for selective compile */
            INPUT plNoDependencies, /* exclude dependencies? */
            OUTPUT iCompSize).

        cError = RETURN-VALUE.
        IF cError <> "" THEN
        DO:
            UNDO $SELECTIVE-COMPILE, RETRY $SELECTIVE-COMPILE.
        END.

        SUBSCRIBE TO "evRtbCancelCompile":U ANYWHERE
            RUN-PROCEDURE "cancel_compile":U.

        COMPILER:MULTI-COMPILE = YES.

        RUN process_compile_list (plXref).

        COMPILER:MULTI-COMPILE = NO.

        IF lHasCompileMsg THEN
        DO:
            cError = "Compile Error." + CHR(1) + cErrorFile.
        END.

        IF lHasXrefError THEN
        DO:
            cError = "Xref Error." + CHR(1) + cErrorFile.
        END.

        /* 10/07/2013 - Thomas Hansen / appSolutions  */
        IF NOT lHasCompileMsg AND
            NOT lHasXrefError AND
            glHasCompileResults THEN
        DO:
            cError = "Compile results." + CHR(1) + cErrorFile.
        END.
        /* 10/07/2013 - Thomas Hansen / appSolutions - End */

        IF lHasCompileMsg OR lHasXrefError OR glHasCompileResults THEN
            RETURN cError.
        ELSE IF lCompileCancelled THEN
                RETURN "Cancel":U.

    END.

    RETURN.

END PROCEDURE.

PROCEDURE build_compile_list PRIVATE:
    /*----------------------------------------------------------------
     Purpose:    Construct a temp-table of objects that require
                 compilation
     Parameters: See 'rtb_selective_compile' for parameter description
     Notes:
    -----------------------------------------------------------------*/
    DEFINE INPUT  PARAMETER piTaskNum        AS INTEGER     NO-UNDO.
    DEFINE INPUT  PARAMETER pcModule         AS CHARACTER   NO-UNDO.
    DEFINE INPUT  PARAMETER pcObjGroup       AS CHARACTER   NO-UNDO.
    DEFINE INPUT  PARAMETER pcObjType        AS CHARACTER   NO-UNDO.
    DEFINE INPUT  PARAMETER pcObject         AS CHARACTER   NO-UNDO.
    DEFINE INPUT  PARAMETER pcPmod           AS CHARACTER   NO-UNDO.
    DEFINE INPUT  PARAMETER plXref           AS LOGICAL     NO-UNDO.
    DEFINE INPUT  PARAMETER plListings       AS LOGICAL     NO-UNDO.
    DEFINE INPUT  PARAMETER plForce          AS LOGICAL     NO-UNDO.
    DEFINE INPUT  PARAMETER plSelective      AS LOGICAL     NO-UNDO.
    DEFINE INPUT  PARAMETER plIgnoreWarnings AS LOGICAL     NO-UNDO.
    DEFINE INPUT  PARAMETER plTaskCompletion AS LOGICAL     NO-UNDO.
    DEFINE INPUT  PARAMETER plLab            AS LOGICAL     NO-UNDO.
    DEFINE INPUT  PARAMETER plNoDependencies AS LOGICAL     NO-UNDO.
    DEFINE OUTPUT PARAMETER piSel            AS INTEGER     NO-UNDO.

    DEFINE VARIABLE hNull AS HANDLE NO-UNDO.

    /***DIAG***/
    MESSAGE {&line-number} PROGRAM-NAME(1) SKIP
        "HERE...XXXX" skip
        DYNAMIC-FUNCTION('fnRtbGetTempDir':U)
        VIEW-AS ALERT-BOX INFO BUTTONS OK.

    RUN rtb/proxy/p/rtbCompilePre.p ON hServer
        (INPUT DYNAMIC-FUNCTION('fnRtbGetTempDir':U),  /* client temp dir */
        INPUT cWorkspace,
        INPUT piTaskNum,
        INPUT pcModule,          /* module */
        INPUT pcObjGroup,        /* group */
        INPUT pcObjType,
        INPUT pcObject,
        INPUT pcPmod,            /* product module */
        INPUT plXref,            /* xref */
        INPUT plListings,        /* listings */
        INPUT plForce,           /* force */
        INPUT plSelective,       /* selective */
        INPUT plIgnoreWarnings,  /* ignore warnings */
        INPUT plTaskCompletion,  /* task completion */
        INPUT NO,                /* OEA style params */
        INPUT lTaskRcode,        /* dependent r-code in task dir */
        INPUT plLab,
        INPUT plNoDependencies,  /* exclude dependencies? */
        INPUT TABLE-HANDLE hNull,
        OUTPUT piSel,
        OUTPUT TABLE ttCompObj,
        OUTPUT cError).

    IF cError <> "" THEN
        RETURN ERROR cError.

    /*
     Let selective compile know the number of objects selected and the actual
     number of entries in the compile list (this isn't necessarily the same
     as the object count).
    */
    IF lSelectiveComp THEN
    DO:
        PUBLISH "evRtbSetCompileObjectsSelected":U (piSel).
        PUBLISH "evRtbSetCompileNumItems":U (getTableCount(TEMP-TABLE ttCompObj:HANDLE)).
    END.

    RUN split_compile_list. /* break into server and client */

    EMPTY TEMP-TABLE ttCompObj. /* done with this so get rid of it */

END PROCEDURE.

PROCEDURE process_compile_list PRIVATE:
    /*----------------------------------------------------------------
     Purpose:    Compile objects in the server and client compilable
                 lists
     Parameters: INPUT
                   plXref - yes = generate xref data
     Notes:
    -----------------------------------------------------------------*/
    DEFINE INPUT PARAMETER plXref AS LOGICAL NO-UNDO.

    DEFINE VARIABLE lHasClientCompiles AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE cTimeStampString   AS CHARACTER NO-UNDO.

    ASSIGN
        cTimeStampString = STRING(DATETIME(TODAY, MTIME))
        cTimeStampString = REPLACE(cTimeStampString, "/":U, "":U)
        cTimeStampString = REPLACE(cTimeStampString, " ":U, "":U)
        cTimeStampString = REPLACE(cTimeStampString, ":":U, "":U)
        cTimeStampString = REPLACE(cTimeStampString, ".":U, "":U).

    cErrorFile = DYNAMIC-FUNCTION('fnRtbGetTempDir') + SUBSTITUTE("compile_&1.log", cTimeStampString).
    DYNAMIC-FUNCTION('fnRtbDeleteOSFile',cErrorFile).

    DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

    $PROCESS-COMPILE-LIST:
    DO ON ERROR UNDO, RETRY:

        IF RETRY THEN
        DO:
            IF cError = "" THEN
                cError = (IF RETURN-VALUE <> "":U THEN
                    RETURN-VALUE
                    ELSE
                    DYNAMIC-FUNCTION('fnRtbErrorHandler',ERROR-STATUS)).
            RETURN ERROR cError.
        END.

        /*
         Go ahead and connect the databases (if needed) to improve performance for
         client/server sessions.
        */
        lHasClientCompiles = CAN-FIND(FIRST ttClientCompilables).

        IF lHasClientCompiles THEN
        DO:
            RUN connect_databases.
            RUN write_task_rcode_xml.
        END.

        DO WHILE CAN-FIND(FIRST ttClientCompilables) AND NOT lCompileCancelled:
            RUN get_batch (TEMP-TABLE ttClientCompilables:HANDLE).
            RUN compile_batch (SESSION:HANDLE).
            RUN xref_batch.
        END.

        /*
         Process server-side compilations
        */
        DO WHILE CAN-FIND(FIRST ttServerCompilables) AND NOT lCompileCancelled:
            RUN get_batch (TEMP-TABLE ttServerCompilables:HANDLE).
            RUN compile_batch (hServer).
            RUN xref_batch.
        END.

        IF lHasClientCompiles THEN
            RUN disconnect_databases.

        IF lCompileCancelled THEN LEAVE $PROCESS-COMPILE-LIST.

    END. /* $process.. */

    RETURN.

END PROCEDURE.

PROCEDURE split_compile_list PRIVATE:
    /*----------------------------------------------------------------
     Purpose:    Split compile list into client and server compilables
     Parameters:
     Notes:
    -----------------------------------------------------------------*/

    FOR EACH ttCompObj:

        IF ttCompObj.client-compile = YES THEN
        DO:
            CREATE ttClientCompilables.
            BUFFER-COPY ttCompObj TO ttClientCompilables.
        END.
        ELSE
        DO:
            CREATE ttServerCompilables.
            BUFFER-COPY ttCompObj TO ttServerCompilables.
        END.

    END.

    RETURN.

END PROCEDURE.

PROCEDURE upload_file PRIVATE:
    /*------------------------------------------------------------------------------
      Purpose: Upload xref, list, or debug file to server.
      Notes:
    ------------------------------------------------------------------------------*/
    DEFINE INPUT PARAMETER pcSource AS CHARACTER NO-UNDO.
    DEFINE INPUT PARAMETER pcTarget AS CHARACTER NO-UNDO.

    DEFINE VARIABLE hFile AS HANDLE NO-UNDO.
    DEFINE VARIABLE hNull AS HANDLE NO-UNDO.

    RUN load_file_into_tt (pcSource, NO, OUTPUT cError, OUTPUT TABLE-HANDLE hFile).

    RUN rtb/proxy/p/rtbSendOSFile.p ON hServer (pcTarget, TABLE-HANDLE hFile, TABLE-HANDLE hNull, OUTPUT cError).

    FINALLY:
        DELETE OBJECT hFile NO-ERROR.
    END.

END PROCEDURE.

PROCEDURE write_task_rcode_xml PRIVATE :
    /*------------------------------------------------------------------------------
      Purpose: If creating Task r-code for dependents then export client compile
               records to XML in the Task folder.
      Notes:   XML file will be used to delete dependent r-code from the Task
               folder when the an object is removed from the Task folder.
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cTaskDir AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cXmlFile AS CHARACTER NO-UNDO.
    DEFINE VARIABLE hTaskSDO AS HANDLE    NO-UNDO.

    IF lTaskRcode THEN
    DO:

        FIND FIRST ttClientCompilables NO-LOCK.

        ASSIGN
            hTaskSDO = fnRtbStartSuperSDO("rtb_q_task.w":U, THIS-PROCEDURE, NO)
            cTaskDir = DYNAMIC-FUNCTION('fnRtbGetTaskDirectory' IN hTaskSDO, ttClientCompilables.task-num)
            cXmlFile = SUBSTITUTE("&1/.&2-&3.xml", cTaskDir, cObject, ttClientCompilables.task-num).

        TEMP-TABLE ttClientCompilables:WRITE-XML("FILE", cXmlFile).

        fnRtbEndSuperProc(THIS-PROCEDURE, hTaskSDO, YES).

    END.

END PROCEDURE.

PROCEDURE xref_batch PRIVATE:
    /*----------------------------------------------------------------
     Purpose:    Generate xref data for the current batch
     Parameters:
     Notes:      If xref errors exists, they will be in a .xe file
                 in the workspace directory. This routine will import
                 the contents and add to our compile.log file.
    -----------------------------------------------------------------*/
    DEFINE VARIABLE cLogFile  AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cXrefText AS CHARACTER NO-UNDO.

    IF NOT TEMP-TABLE ttCompObj:HAS-RECORDS THEN RETURN.

    PROCESS EVENTS.

    RUN rtb/proxy/p/rtbCompilePost.p ON hServer (INPUT-OUTPUT TABLE ttCompObj).

    FOR EACH ttCompObj:

        IF ttCompObj.xref-errors <> "" THEN
        DO:

            OUTPUT TO VALUE(cErrorFile) APPEND.

            PUT UNFORMATTED
                "Object: " ttCompObj.object SKIP.
            PUT UNFORMATTED
                "Pmod  : " ttCompObj.pmod SKIP.

            IF ttCompObj.xref-errors BEGINS "Xref Error":U THEN
            DO: /* log file */

                cLogFile = REPLACE(ttCompObj.xref-errors,"Xref Error Log: ":U, "":U).
                INPUT FROM VALUE(cLogFile).

                REPEAT:
                    IMPORT UNFORMATTED cXrefText.
                    PUT UNFORMATTED cXrefText SKIP.
                END.

                INPUT CLOSE.

            END.
            ELSE
            DO:
                PUT UNFORMATTED ttCompObj.xref-errors SKIP(1).
            END.

            OUTPUT CLOSE.

            lHasXrefError = YES.

        END.

    END.

    EMPTY TEMP-TABLE ttCompObj.

END PROCEDURE.

PROCEDURE get_batch PRIVATE:
    /*----------------------------------------------------------------
     Purpose:    Get a batch of records to compile based on the batch
                 size
     Parameters: INPUT
                   phTT - handle of TT from which to create a batch
     Notes:      phTT will be a handle to either ttClientCompilables
                 or ttServerComilables.
    -----------------------------------------------------------------*/
    DEFINE INPUT PARAMETER phTT AS HANDLE NO-UNDO.

    DEFINE VARIABLE hQry AS HANDLE  NO-UNDO.
    DEFINE VARIABLE hBuf AS HANDLE  NO-UNDO.
    DEFINE VARIABLE i    AS INTEGER NO-UNDO.

    EMPTY TEMP-TABLE ttCompBatch. /* remove any existing batch */

    IF iBatchSize = 0 THEN
        iBatchSize = iCompSize.
    ELSE
        iBatchSize = MIN(iBatchSize, iCompSize).

    hBuf = phTT:DEFAULT-BUFFER-HANDLE.

    CREATE QUERY hQry.
    hQry:FORWARD-ONLY = YES.
    hQry:SET-BUFFERS(hBuf).
    hQry:QUERY-PREPARE("for each " + phTT:NAME).

    hQry:QUERY-OPEN().
    hQry:GET-FIRST().
    DO WHILE NOT hQry:QUERY-OFF-END AND i < iBatchSize:
        CREATE ttCompBatch.
        TEMP-TABLE ttCompBatch:DEFAULT-BUFFER-HANDLE:BUFFER-COPY(hBuf).
        hBuf:BUFFER-DELETE.
        i = i + 1.
        hQry:GET-NEXT().
    END.

    DELETE OBJECT hQry NO-ERROR.

    RETURN.

END PROCEDURE.

PROCEDURE compile_batch PRIVATE:
    /*----------------------------------------------------------------
     Purpose:    Compile a batch of compilation records
     Parameters: INPUT
                   phServer - SESSION or Roundtable appServer
     Notes:
    -----------------------------------------------------------------*/
    DEFINE INPUT PARAMETER phServer AS HANDLE NO-UNDO.

    DEFINE VARIABLE lError               AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE lWarn                AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE cCompileresultString AS CHARACTER NO-UNDO.

    PROCESS EVENTS.

    RUN rtb/proxy/p/rtbCompileObjects.p ON phServer
        (INPUT cWorkspace,
        INPUT-OUTPUT TABLE ttCompBatch,
        OUTPUT cError).

    IF cError <> "" THEN
        RETURN ERROR cError.

    /*
     Analyze results
    */
    FOR EACH ttCompBatch:

        IF lSelectiveComp THEN
            PUBLISH "evRtbSetCompileObject":U (ttCompBatch.object).

        ASSIGN
            lError = (ttCompBatch.comp-status = "COMPILE-ERROR":U)
            lWarn  = (ttCompBatch.warning-found = TRUE).

        IF (lError OR (lWarn AND NOT lIgnoreWarn)) THEN
        DO:

            IF lSelectiveComp THEN
                PUBLISH "evRtbCompileFailed":U.

            OUTPUT TO VALUE(cErrorFile) APPEND.

            PUT UNFORMATTED
                "Object: " ttCompBatch.object SKIP.
            PUT UNFORMATTED
                "Pmod  : " ttCompBatch.pmod SKIP.
            PUT UNFORMATTED
                "Path  : " ttCompBatch.obj-prog SKIP.
            PUT UNFORMATTED
                "Error : " ttCompBatch.comp-errors SKIP(1).

            OUTPUT CLOSE.

            lHasCompileMsg = YES.

            IF lStopOnError AND NOT lWarn THEN RUN cancel_compile.

            IF lError THEN NEXT.

        END.
        /* 10/07/2013 - Thomas Hansen / appSolutions
                        Optional output for showing which objects have been compiled in addition to the ones with errors*/
        ELSE
            IF glOutputSuccessfulCompiles THEN
            DO:
                OUTPUT TO VALUE(cErrorFile) APPEND.

                ASSIGN
                    ttCompBatch.comp-status = "INFO":U
                    lHasCompileMsg          = NO
                    cCompileresultString    = SUBSTITUTE("Object: &1 - OK~n" +
                                                      "  Pmod: &2~n" +
                                                      "  Path: &3~n",
                                                      ttCompBatch.object,
                                                      ttCompBatch.pmod,
                                                      ttCompBatch.obj-prog).
                PUT UNFORMATTED cCompileresultString SKIP.

                ASSIGN
                    glHasCompileResults = TRUE.

                OUTPUT CLOSE.
            END.
        /* 10/07/2013 - Thomas Hansen / appSolutions - End */

        /*
         Copy record to "safe" list
        */
        CREATE ttCompObj.
        BUFFER-COPY ttCompBatch TO ttCompObj.

        /*
         Send xref file to server if needed (perhaps I can just check the value of hServer?)
        */
        IF ttCompObj.client-compile = YES
            AND ttCompObj.comp-status = "COMPILED":U
            AND (ttCompObj.xref-client-tmp <> ""
            AND ttCompObj.xref-client-tmp <> ttCompObj.xref-server-tmp) THEN
        DO:

            RUN upload_file (ttCompObj.xref-client-tmp, ttCompObj.xref-server-tmp).

        END.

        /*
         Send list file to server if needed
        */
        IF ttCompObj.client-compile = YES
            AND ttCompObj.comp-status = "COMPILED":U
            AND (ttCompObj.list-client-tmp <> ""
            AND ttCompObj.list-client-tmp <> ttCompObj.list-server-tmp) THEN
        DO:

            RUN upload_file (ttCompObj.list-client-tmp, ttCompObj.list-server-tmp).

        END.

        /*
         Send debug file to server if needed
        */
        IF ttCompObj.client-compile = YES
            AND ttCompObj.comp-status = "COMPILED":U
            AND (ttCompObj.debug-client-tmp <> ""
            AND ttCompObj.debug-client-tmp <> ttCompObj.debug-server-tmp) THEN
        DO:

            RUN upload_file (ttCompObj.debug-client-tmp, ttCompObj.debug-server-tmp).

        END.

        IF lSelectiveComp THEN
            PUBLISH "evRtbCompileComplete":U.

    END. /* each ttCOmpObj */

END PROCEDURE.

PROCEDURE connect_databases PRIVATE :
    /*------------------------------------------------------------------------------
      Purpose:     Connect to databases for specified Configuration
      Parameters:
      Notes:
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

    DEFINE VARIABLE iSid   AS INTEGER   NO-UNDO.

    RUN client_connect_workspace_dbs
        (INPUT  "",            /* prev configuration */
        INPUT  cWorkspace,    /* selected configuration */
        OUTPUT cError).

    IF cError <> "":U THEN
        RETURN ERROR cError.
    ELSE
        RETURN.

END PROCEDURE.

PROCEDURE disconnect_databases PRIVATE :
    /*------------------------------------------------------------------------------
      Purpose:     Disconnect Configuration databases previously connetected in
                   connect_databases
      Parameters:
      Notes:       Specifying no workspace as the selected workspace and
                   the current workspace as the previous workspace will
                   effectively disconnect the databases for the current
                   workspace
    ------------------------------------------------------------------------------*/
    DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

    /*
     Is the user's preference to NOT connect workspace databases?
    */
    IF DYNAMIC-FUNCTION('fnRtbGetRegistryValue':U,"Databases":U,"ConnectWorkspaceDBs":U) = "No":U THEN

        RUN client_connect_workspace_dbs
            (INPUT cWorkspace,            /* prev workspace */
            INPUT "",                    /* selected workspace */
            OUTPUT cError).

    IF cError <> "":U THEN
        RETURN ERROR cError.
    ELSE
        RETURN.

END PROCEDURE.

PROCEDURE cancel_compile:
    lCompileCancelled = YES.
    RETURN.
END PROCEDURE.

PROCEDURE DESTRUCTOR:

    /*    IF VALID-HANDLE(hServer) THEN                        */
    /*        RUN appServerDisconnect("roundtable":U) NO-ERROR.*/

    fnRtbEndSuperProc(THIS-PROCEDURE, hDbparam, YES).
    fnRtbEndSuperProc(THIS-PROCEDURE, hFileLW, YES).

    DELETE PROCEDURE THIS-PROCEDURE.

    RETURN.

END PROCEDURE.

FUNCTION getTableCount RETURNS INTEGER PRIVATE
    ( INPUT phTT AS HANDLE ):
    /*----------------------------------------------------------------
      Purpose:    Count the records for a given temp-table handle
      Parameters: INPUT
                   phTT - temp-table handle
    -----------------------------------------------------------------*/
    DEFINE VARIABLE iCnt AS INTEGER NO-UNDO.
    DEFINE VARIABLE hQry AS HANDLE  NO-UNDO.
    DEFINE VARIABLE hBuf AS HANDLE  NO-UNDO.

    hBuf = phTT:DEFAULT-BUFFER-HANDLE.

    CREATE QUERY hQry.
    hQry:SET-BUFFERS(hBuf).
    hQry:QUERY-PREPARE("preselect each " + phTT:NAME).
    hQry:QUERY-OPEN().

    iCnt = hQry:NUM-RESULTS.

    DELETE OBJECT hQry.

    RETURN iCnt.

END FUNCTION.
